angular.module('futd', ['ionic', 'ngCordova', 'ngMessages', 'ngSanitize', 'angular.filter', 'angularMoment', 'base64', 'futd.filters', 'futd.controllers', 'futd.services', 'futd.models'])

.run(function(ionPlatform, $rootScope, $ionicLoading, $state) {
  ionPlatform.ready.then(function() {
    if(window.StatusBar) {
      // org.apache.cordova.statusbar required
      StatusBar.styleDefault();
    }
    if (window.analytics) {
      window.analytics.startTrackerWithId('UA-27472118-3');
    }
  });

  $rootScope.$on('loading:show', function() {
    $ionicLoading.show({
      template: 'Loading',
      animation: 'fade-in',
      showBackdrop: true,
      maxWidth: 200,
      showDelay: 0
    });
  });

  $rootScope.$on('loading:hide', function() {
    $ionicLoading.hide();
  });

  $rootScope.$on('locked:go', function() {
    $state.go('locked');
  });
})

.config(function($httpProvider, $stateProvider, $urlRouterProvider, $sceDelegateProvider) {

  $httpProvider.interceptors.push(function($q, $rootScope) {
    return {
      request: function(config) {
        if (config && config.url && config.url.startsWith("http://")) {
          $rootScope.$broadcast('loading:show');
        }
        return config || $q.when(config);
      },
      requestError: function(request) {
        $rootScope.$broadcast('loading:hide');
        return $q.reject(request);
      },
      response: function(response) {
        if (response && response.config && response.config.url && response.config.url.startsWith("http://")) {
          $rootScope.$broadcast('loading:hide');
        }
        return response || $q.when(response);
      },
      responseError: function(response) {
        console.log(response);
        if (response && response.config && response.config.url && response.config.url.startsWith("http://")) {
          $rootScope.$broadcast('loading:hide');
        }
        if (response && response.status === 403 && response.data && response.data.error && response.data.error === 1000) {
          $rootScope.$broadcast('locked:go');
        }
        return $q.reject(response);
      }
    };
  });

  $stateProvider
    .state('loading', {
      url: '/loading',
      templateUrl: 'templates/loading.html',
      controller: 'LoadingController'
    })
    .state('signin', {
      cache: false,
      url: '/signin',
      templateUrl: 'templates/signin.html',
      controller: 'SigninController'
    })
    .state('about', {
      url: '/about',
      templateUrl: 'templates/about.html',
      controller: 'AboutController'
    })
    .state('locked', {
      url: '/locked',
      templateUrl: 'templates/locked.html'
    })
    .state('tabs', {
      url: '/tab',
      abstract: true,
      templateUrl: 'templates/tabs.html'
    })
    .state('tabs.matches', {
      cache: false,
      url: '/matches',
      views: {
        'matches-tab': {
          templateUrl: 'templates/matches.html',
          controller: 'MatchesController'
        }
      }
    })
    .state('tabs.match-detail', {
      cache: false,
      url: '/matches/:matchId',
      params: {tipId: null},
      views: {
        'matches-tab': {
          templateUrl: 'templates/match-detail.html',
          controller: 'MatchDetailController'
        }
      }
    })
    .state('tabs.live', {
      cache: false,
      url: '/live',
      views: {
        'live-tab': {
          templateUrl: 'templates/live.html',
          controller: 'LiveController'
        }
      }
    })
    .state('tabs.live-match-detail', {
      cache: false,
      url: '/live/:matchId',
      views: {
        'live-tab': {
          templateUrl: 'templates/match-detail.html',
          controller: 'MatchDetailController'
        }
      }
    })
    .state('tabs.tipsters', {
      cache: false,
      url: '/tipsters',
      views: {
        'tipsters-tab': {
          templateUrl: 'templates/tipsters.html',
          controller: 'TipstersController'
        }
      }
    })
    .state('tabs.tipster-detail', {
      cache: false,
      url: '/tipsters/:tipsterId',
      views: {
        'tipsters-tab': {
          templateUrl: 'templates/tipster-detail.html',
          controller: 'TipsterDetailController'
        }
      }
    })
    .state('tabs.notifications', {
      cache: false,
      url: '/notifications',
      params: {notificationId: null},
      views: {
        'notifications-tab': {
          templateUrl: 'templates/notifications.html',
          controller: 'NotificationsController'
        }
      }
    })
    .state('tabs.account', {
      cache: false,
      url: '/account',
      views: {
        'account-tab': {
          templateUrl: 'templates/account.html',
          controller: 'AccountController'
        }
      }
    });

  $urlRouterProvider.otherwise('/loading');

  moment.locale('en', {
    calendar : {
      lastDay : '[Yesterday]',
      sameDay : '[Today]',
      nextDay : '[Tomorrow]',
      lastWeek : '[last] dddd',
      nextWeek : 'dddd',
      sameElse : 'L'
    }
  });

  $sceDelegateProvider.resourceUrlWhitelist([
    'self',
    new RegExp('^(http[s]?):\/\/(w{3}.)?youtube-nocookie\.com/.+$')]);

})

.value('Settings', {timezone: 'CET'})

.directive("futdHeader", function() {
  return {
    restrict: "E",
    templateUrl: "templates/header.html"
  }
})

.directive("futdBanner", function() {
  return {
    restrict: "E",
    scope: { ad: '=' },
    templateUrl: "templates/banner.html"
  }
})

.directive("futdMatch", function() {
  return {
    restrict: "E",
    scope: { match: '=' },
    templateUrl: "templates/match.html"
  }
})

.directive("futdTip", function() {
  return {
    restrict: "E",
    scope: { tip: '=', highlight: '=', includematch: '=' },
    templateUrl: "templates/tip.html"
  }
})

.directive("futdNotificationText", function() {
  return {
    restrict: "E",
    scope: { notification: '=' },
    templateUrl: "templates/notification-text.html"
  }
})

.directive("futdNotificationYoutube", function() {
  return {
    restrict: "E",
    scope: { notification: '=' },
    templateUrl: "templates/notification-youtube.html"
  }
})

.directive("futdNotificationSpelpodden", function() {
  return {
    restrict: "E",
    scope: { notification: '=' },
    templateUrl: "templates/notification-spelpodden.html"
  }
})