controllers = angular.module('futd.controllers', []);

controllers.controller('FutdController', function(ionPlatform, $scope, $state, $cordovaPush, $cordovaToast, $cordovaDialogs, Devices) {
  $scope.platform = ionic.Platform.platform();

  $scope.registerPushNotifications = function() {
    if(ionic.Platform.isWebView()) {
      console.log('Starting push notification registration process');
    } else {
      console.log('Skip registering for push notifications when running in web browser');
      return;
    }

    var config = null;

    if (ionic.Platform.isIOS()) {
      config = {
        "badge": "false",
        "sound": "true",
        "alert": "true"
      }
    } else if (ionic.Platform.isAndroid()) {
      config = {
        "senderID": "205040381563"
      };
    } else {
      return;
    }

    $cordovaPush.register(config).then(function(token) {
      if (ionic.Platform.isIOS()) {
        Devices.registerAppleDevice(token, function(success) {
          if (success && success == true) {
            // $cordovaToast.showShortCenter("Stored device token ("+token+") on server");
          } else {
            // $cordovaToast.showShortCenter("Failed to store device token ("+token+")");
          }
        });
      } else if(ionic.Platform.isAndroid()) {
        // $cordovaToast.showShortCenter("Register success");
      }
    }, function(err) {
      // console.log("Register error " + err);
      // $cordovaToast.showShortCenter("Register error " + err);
    });
  }

  $scope.$on('$cordovaPush:notificationReceived', function(event, notification) {
    // console.log(JSON.stringify([notification]));

    ionPlatform.ready.then(function() {
      // $cordovaToast.showLongTop(JSON.stringify([notification]));

      // Only handles iOS for now, see sample project for handles Android
      // https://github.com/hollyschinsky/PushNotificationSample
      if (ionic.Platform.isIOS()) {
        handleIOS(notification);
      } else if (ionic.Platform.isAndroid()) {
        handleAndroid(notification);
      }
    });
  });

  function handleIOS(notification) {
    message = {
      matchId: notification.matchId,
      tipId: notification.tipId,
      isLive: notification.isLive,
      notificationId: notification.notificationId,
      title: notification.title,
      text: notification.alert
    }
    showMessage(message, notification.foreground == "1");
  }

  function handleAndroid(notification) {
    switch(notification.event) {
      case 'registered':
        if (notification.regid.length > 0) {
          // $cordovaToast.showShortCenter("GCM regid = " + notification.regid);
          Devices.registerAndroidDevice(notification.regid, function(success) {
            if (success && success == true) {
              // $cordovaToast.showShortCenter("Stored regid ("+notification.regid+") on server");
            } else {
              // $cordovaToast.showShortCenter("Failed to store regid ("+notification.regid+")");
            }
          });
        } else {
          $cordovaToast.showShortCenter("Invalid GCM regid received");
        }
        break;
      case 'message':
        // $cordovaToast.showShortCenter("GCM message received - " + notification.payload.message);
        message = {
          matchId: notification.payload.matchId,
          tipId: notification.payload.tipId,
          isLive: notification.payload.isLive,
          notificationId: notification.payload.notificationId,
          title: notification.payload.title,
          text: notification.payload.message
        }
        showMessage(message, notification.foreground == "1");
        break;
      case 'error':
        $cordovaToast.showShortCenter("GCM error = " + notification.msg);
        break;
      default:
        $cordovaToast.showShortCenter("Unknown GCM event");
        break;
    }
  }

  function showMessage(message, foreground) {
    if (message.matchId && message.tipId || message.notificationId) {
      if (message.notificationId) {
        target = 'tabs.notifications';
        options = {notificationId: message.notificationId};
        title = message.title || 'New notification';
        buttons = ['Close', 'Show'];
      } else {
        target = message.isLive ? 'tabs.live' : 'tabs.match-detail';
        options = message.isLive ? {tipId: message.tipId} : {matchId: message.matchId, tipId: message.tipId};
        title = 'New tip posted';
        buttons = ['Close', 'Show'];
      }

      if (foreground) {
        $cordovaDialogs.confirm(message.text, title, buttons)
          .then(function(buttonIndex) {
            if (buttonIndex == 2) {
              $state.go(target, options);
            }
          });
      } else {
        $state.go(target, options);
      }
    } else {
      title = message.title || "Push notification received";
      $cordovaDialogs.alert(message.text, title);
    }
  }
});

controllers.controller('LoadingController', function($scope, $state, Users) {
  $scope.$on('$ionicView.enter', function() {
    Users.currentUser(function(user) {
      if (user) {
        $scope.currentUser = user;
        $scope.registerPushNotifications();
        $state.go('tabs.matches');
      }
    });
  });
})

controllers.controller('SigninController', function($scope, $ionicModal, $state, Api, Users, $localstorage) {
  if (window.analytics) {
    window.analytics.trackView('Signin');
  }

  $scope.disableSignIn = false;
  $scope.credentials = {username: '', password: ''};
  $scope.signinRequest = {failed: false, error: false};

  $scope.signIn = function(form) {
    if (form.$valid) {
      $scope.disableSignIn = true;
      Api.accessToken(
        $scope.credentials.username,
        $scope.credentials.password,
        function(success, data) {
          if (success) {
            hasSeenIntroduction = $localstorage.get('hasSeenIntroduction');
            if (ionic.Platform.isAndroid() || (hasSeenIntroduction && hasSeenIntroduction == 'true')) {
              $scope.registerPushNotifications();
              $state.go('tabs.matches');
            } else {
              Users.currentUser(function(user) {
                if (user) {
                  $scope.currentUser = user;
                  $localstorage.set('hasSeenIntroduction', 'true');
                  $scope.openModal();
                } else {
                  $scope.signinRequest.error = true;
                }
              });
            }
          } else {
            console.log(data);
            if (data && data.error && data.error == "invalid_grant") {
              $scope.signinRequest.failed = true;
            } else {
              $scope.signinRequest.error = true;
            }
          }

          $scope.disableSignIn = false;
        }
      );
    }
  };

  $ionicModal.fromTemplateUrl('templates/introduction.html', {
    scope: $scope,
    animation: 'slide-in-up'
  }).then(function(modal) {
    $scope.modal = modal;
  });
  $scope.openModal = function() {
    if (window.analytics) {
      window.analytics.trackView('Introduction');
    }
    $scope.modal.show();
  };
  $scope.closeModal = function() {
    $scope.registerPushNotifications();
    $scope.modal.hide();
    $state.go('tabs.matches');
  };

  $scope.$on('$destroy', function() {
    $scope.modal.remove();
  });
  $scope.$on('modal.hidden', function() {
  });
  $scope.$on('modal.removed', function() {
  });
});

controllers.controller('AboutController', function($scope) {
  if (window.analytics) {
    window.analytics.trackView('About');
  }
});

controllers.controller('NotificationsController', function($scope, $stateParams, Notifications, Users) {
  Users.currentUser(function(user) {
    if (user) { $scope.currentUser = user; }
  });

  if (window.analytics) {
    window.analytics.trackView('Notifications');
  }

  if ($stateParams.notificationId) {
    $scope.highlightedNotificationId = parseInt($stateParams.notificationId);
  }

  $scope.$on('$ionicView.enter', function() {
    Notifications.list(function(notifications) {
      $scope.notifications = notifications;
    });
  });
});

controllers.controller('MatchesController', function($scope, Matches, Users) {
  Users.currentUser(function(user) {
    if (user) { $scope.currentUser = user; }
  });

  if (window.analytics) {
    window.analytics.trackView('Matches');
  }

  $scope.$on('$ionicView.enter', function() {
    Matches.groupByDate(function(items) {
      $scope.items = items;
    });
  });

  $scope.refreshMatches = function() {
    Matches.groupByDate(function(items) {
      $scope.items = items;
      $scope.$broadcast('scroll.refreshComplete');
    })
  }
});

controllers.controller('MatchDetailController', function($scope, $state, $stateParams, Matches, Users) {
  Users.currentUser(function(user) {
    if (user) { $scope.currentUser = user; }
  });

  if (window.analytics) {
    window.analytics.trackView('Match Details');
  }

  $scope.isLive = function() {
    return $state.is('tabs.live-match-detail');
  }

  $scope.$on('$ionicView.enter', function() {
    Matches.get($stateParams.matchId, function(match) {
      $scope.match = match;
    });

    if ($stateParams.tipId) {
      $scope.highlightedTipId = parseInt($stateParams.tipId);
    }
  });

  $scope.refreshMatch = function() {
    Matches.get($stateParams.matchId, function(match) {
      $scope.match = match;
      $scope.$broadcast('scroll.refreshComplete');
    });
  }
});

controllers.controller('LiveController', function($scope, $stateParams, Matches, Tips, Users) {
  Users.currentUser(function(user) {
    if (user) { $scope.currentUser = user; }
  });

  if (window.analytics) {
    window.analytics.trackView('Live');
  }

  $scope.$on('$ionicView.enter', function() {
    Tips.live(function(tips) {
      $scope.tips = tips;

      Matches.live(function(matches) {
        $scope.matches = matches;
      });
    });

    if ($stateParams.tipId) {
      $scope.highlightedTipId = parseInt($stateParams.tipId);
    }
  });

  $scope.refreshContent = function() {
    Tips.live(function(tips) {
      $scope.tips = tips;

      Matches.live(function(matches) {
        $scope.matches = matches;
        $scope.$broadcast('scroll.refreshComplete');
      });
    });
  }
});

controllers.controller('TipstersController', ['$scope', 'Tipsters', function($scope, Tipsters) {
  if (window.analytics) {
    window.analytics.trackView('Tipsters');
  }

  $scope.mode = 'top';

  Tipsters.top(function(tipsters) {
    $scope.tipsters = tipsters;
  });

  $scope.showTop = function() {
    $scope.mode = 'top';
    Tipsters.top(function(tipsters) {
      $scope.tipsters = tipsters;
    });
  }

  $scope.showAll = function() {
    $scope.mode = 'all';
    Tipsters.all(function(tipsters) {
      $scope.tipsters = tipsters;
    });
  }
}]);

controllers.controller('TipsterDetailController', ['$scope', '$stateParams', 'Tipsters', function($scope, $stateParams, Tipsters) {
  $scope.$on('$ionicView.enter', function() {
    Tipsters.get($stateParams.tipsterId, function(tipster) {
      $scope.tipster = tipster;

      if (window.analytics) {
        window.analytics.trackView('Tipster - ' + $scope.tipster.id);
      }
    });
  });
}]);

controllers.controller('AccountController', function($scope, $state, Api, Users) {
  Users.currentUser(function(user) {
    if (user) {
      $scope.currentUser = user;
      $scope.available_timezones = [{label:"International Date Line West",tz:"Pacific/Midway"},{label:"Midway Island",tz:"Pacific/Midway"},{label:"American Samoa",tz:"Pacific/Pago_Pago"},{label:"Hawaii",tz:"Pacific/Honolulu"},{label:"Alaska",tz:"America/Juneau"},{label:"Pacific Time (US & Canada)",tz:"America/Los_Angeles"},{label:"Tijuana",tz:"America/Tijuana"},{label:"Mountain Time (US & Canada)",tz:"America/Denver"},{label:"Arizona",tz:"America/Phoenix"},{label:"Chihuahua",tz:"America/Chihuahua"},{label:"Mazatlan",tz:"America/Mazatlan"},{label:"Central Time (US & Canada)",tz:"America/Chicago"},{label:"Saskatchewan",tz:"America/Regina"},{label:"Guadalajara",tz:"America/Mexico_City"},{label:"Mexico City",tz:"America/Mexico_City"},{label:"Monterrey",tz:"America/Monterrey"},{label:"Central America",tz:"America/Guatemala"},{label:"Eastern Time (US & Canada)",tz:"America/New_York"},{label:"Indiana (East)",tz:"America/Indiana/Indianapolis"},{label:"Bogota",tz:"America/Bogota"},{label:"Lima",tz:"America/Lima"},{label:"Quito",tz:"America/Lima"},{label:"Atlantic Time (Canada)",tz:"America/Halifax"},{label:"Caracas",tz:"America/Caracas"},{label:"La Paz",tz:"America/La_Paz"},{label:"Santiago",tz:"America/Santiago"},{label:"Newfoundland",tz:"America/St_Johns"},{label:"Brasilia",tz:"America/Sao_Paulo"},{label:"Buenos Aires",tz:"America/Argentina/Buenos_Aires"},{label:"Montevideo",tz:"America/Montevideo"},{label:"Georgetown",tz:"America/Guyana"},{label:"Greenland",tz:"America/Godthab"},{label:"Mid-Atlantic",tz:"Atlantic/South_Georgia"},{label:"Azores",tz:"Atlantic/Azores"},{label:"Cape Verde Is.",tz:"Atlantic/Cape_Verde"},{label:"Dublin",tz:"Europe/Dublin"},{label:"Edinburgh",tz:"Europe/London"},{label:"Lisbon",tz:"Europe/Lisbon"},{label:"London",tz:"Europe/London"},{label:"Casablanca",tz:"Africa/Casablanca"},{label:"Monrovia",tz:"Africa/Monrovia"},{label:"UTC",tz:"Etc/UTC"},{label:"Belgrade",tz:"Europe/Belgrade"},{label:"Bratislava",tz:"Europe/Bratislava"},{label:"Budapest",tz:"Europe/Budapest"},{label:"Ljubljana",tz:"Europe/Ljubljana"},{label:"Prague",tz:"Europe/Prague"},{label:"Sarajevo",tz:"Europe/Sarajevo"},{label:"Skopje",tz:"Europe/Skopje"},{label:"Warsaw",tz:"Europe/Warsaw"},{label:"Zagreb",tz:"Europe/Zagreb"},{label:"Brussels",tz:"Europe/Brussels"},{label:"Copenhagen",tz:"Europe/Copenhagen"},{label:"Madrid",tz:"Europe/Madrid"},{label:"Paris",tz:"Europe/Paris"},{label:"Amsterdam",tz:"Europe/Amsterdam"},{label:"Berlin",tz:"Europe/Berlin"},{label:"Bern",tz:"Europe/Berlin"},{label:"Rome",tz:"Europe/Rome"},{label:"Stockholm",tz:"Europe/Stockholm"},{label:"Vienna",tz:"Europe/Vienna"},{label:"West Central Africa",tz:"Africa/Algiers"},{label:"Bucharest",tz:"Europe/Bucharest"},{label:"Cairo",tz:"Africa/Cairo"},{label:"Helsinki",tz:"Europe/Helsinki"},{label:"Kyiv",tz:"Europe/Kiev"},{label:"Riga",tz:"Europe/Riga"},{label:"Sofia",tz:"Europe/Sofia"},{label:"Tallinn",tz:"Europe/Tallinn"},{label:"Vilnius",tz:"Europe/Vilnius"},{label:"Athens",tz:"Europe/Athens"},{label:"Istanbul",tz:"Europe/Istanbul"},{label:"Minsk",tz:"Europe/Minsk"},{label:"Jerusalem",tz:"Asia/Jerusalem"},{label:"Harare",tz:"Africa/Harare"},{label:"Pretoria",tz:"Africa/Johannesburg"},{label:"Moscow",tz:"Europe/Moscow"},{label:"St. Petersburg",tz:"Europe/Moscow"},{label:"Volgograd",tz:"Europe/Moscow"},{label:"Kuwait",tz:"Asia/Kuwait"},{label:"Riyadh",tz:"Asia/Riyadh"},{label:"Nairobi",tz:"Africa/Nairobi"},{label:"Baghdad",tz:"Asia/Baghdad"},{label:"Tehran",tz:"Asia/Tehran"},{label:"Abu Dhabi",tz:"Asia/Muscat"},{label:"Muscat",tz:"Asia/Muscat"},{label:"Baku",tz:"Asia/Baku"},{label:"Tbilisi",tz:"Asia/Tbilisi"},{label:"Yerevan",tz:"Asia/Yerevan"},{label:"Kabul",tz:"Asia/Kabul"},{label:"Ekaterinburg",tz:"Asia/Yekaterinburg"},{label:"Islamabad",tz:"Asia/Karachi"},{label:"Karachi",tz:"Asia/Karachi"},{label:"Tashkent",tz:"Asia/Tashkent"},{label:"Chennai",tz:"Asia/Kolkata"},{label:"Kolkata",tz:"Asia/Kolkata"},{label:"Mumbai",tz:"Asia/Kolkata"},{label:"New Delhi",tz:"Asia/Kolkata"},{label:"Kathmandu",tz:"Asia/Kathmandu"},{label:"Astana",tz:"Asia/Dhaka"},{label:"Dhaka",tz:"Asia/Dhaka"},{label:"Sri Jayawardenepura",tz:"Asia/Colombo"},{label:"Almaty",tz:"Asia/Almaty"},{label:"Novosibirsk",tz:"Asia/Novosibirsk"},{label:"Rangoon",tz:"Asia/Rangoon"},{label:"Bangkok",tz:"Asia/Bangkok"},{label:"Hanoi",tz:"Asia/Bangkok"},{label:"Jakarta",tz:"Asia/Jakarta"},{label:"Krasnoyarsk",tz:"Asia/Krasnoyarsk"},{label:"Beijing",tz:"Asia/Shanghai"},{label:"Chongqing",tz:"Asia/Chongqing"},{label:"Hong Kong",tz:"Asia/Hong_Kong"},{label:"Urumqi",tz:"Asia/Urumqi"},{label:"Kuala Lumpur",tz:"Asia/Kuala_Lumpur"},{label:"Singapore",tz:"Asia/Singapore"},{label:"Taipei",tz:"Asia/Taipei"},{label:"Perth",tz:"Australia/Perth"},{label:"Irkutsk",tz:"Asia/Irkutsk"},{label:"Ulaanbaatar",tz:"Asia/Ulaanbaatar"},{label:"Seoul",tz:"Asia/Seoul"},{label:"Osaka",tz:"Asia/Tokyo"},{label:"Sapporo",tz:"Asia/Tokyo"},{label:"Tokyo",tz:"Asia/Tokyo"},{label:"Yakutsk",tz:"Asia/Yakutsk"},{label:"Darwin",tz:"Australia/Darwin"},{label:"Adelaide",tz:"Australia/Adelaide"},{label:"Canberra",tz:"Australia/Melbourne"},{label:"Melbourne",tz:"Australia/Melbourne"},{label:"Sydney",tz:"Australia/Sydney"},{label:"Brisbane",tz:"Australia/Brisbane"},{label:"Hobart",tz:"Australia/Hobart"},{label:"Vladivostok",tz:"Asia/Vladivostok"},{label:"Guam",tz:"Pacific/Guam"},{label:"Port Moresby",tz:"Pacific/Port_Moresby"},{label:"Magadan",tz:"Asia/Magadan"},{label:"Solomon Is.",tz:"Pacific/Guadalcanal"},{label:"New Caledonia",tz:"Pacific/Noumea"},{label:"Fiji",tz:"Pacific/Fiji"},{label:"Kamchatka",tz:"Asia/Kamchatka"},{label:"Marshall Is.",tz:"Pacific/Majuro"},{label:"Auckland",tz:"Pacific/Auckland"},{label:"Wellington",tz:"Pacific/Auckland"},{label:"Nuku'alofa",tz:"Pacific/Tongatapu"},{label:"Tokelau Is.",tz:"Pacific/Fakaofo"},{label:"Chatham Is.",tz:"Pacific/Chatham"},{label:"Samoa",tz:"Pacific/Apia"}];
      $scope.selectedTimezone = $scope.available_timezones.filter(function(tz) {return tz.tz == user.timezone})[0];
    }
  });

  if (window.analytics) {
    window.analytics.trackView('Account');
  }

  $scope.saveTimezone = function(tz) {
    var u = {id: $scope.currentUser.id, username: $scope.currentUser.username, email: $scope.currentUser.email, timezone: tz.tz};
    Users.update(u, function(success) {
      if (success) {
        $scope.currentUser.timezone = tz.tz;
      } else {
        console.log('Failed to update timezone');
      };
    });
  }

  $scope.signOut = function() {
    Api.clearTokens();
    $state.go('signin');
  };
});

