models = angular.module('futd.models', []);

models.factory('User', function() {
  function User(data) {
    this.id = data.id;
    this.username = data.username;
    this.email = data.email;
    this.timezone = data.timezone;
    this.lastCheckedNotificationsAt = data.lastCheckedNotificationsAt;
    this.unreadNotifications = data.unreadNotifications;
  }

  return User;
});

models.factory('ListItem', [function() {
  function ListItem(type, inner, grouping, order) {
    this.type = type;
    this.inner = inner;
    this.grouping = grouping;
    this.order = order;
  }

  return ListItem;
}]);

models.factory('Ad', [function() {
  function Ad(url, img) {
    this.url = url;
    this.img = img;
  }

  Ad.prototype.html = function() {
    return "<a href='#' class='item item-stable item-tight text-center' onclick='window.open(\"" + this.url + "\", \"_system\"); return false;'><img src='" + this.img + "' style='height: 40px;' /></a>";
  }

  return Ad;
}])

models.factory('Match', ['Settings', 'Tip', function(Settings, Tip) {
  function Match(id, title, startsAt, competition, tips) {
    this.id = id;
    this.title = title;
    this.startsAt = moment(startsAt).tz(Settings.timezone);
    this.competition = competition;
    if (tips && tips.constructor === Array) {
      this.tips = tips.map(function(tip) { return new Tip(tip); })
    } else {
      this.tips = [];
    }
  }

  Match.prototype.homeTeam = function() {
    return this.title.substring(0, this.title.indexOf(' v '));
  }

  Match.prototype.awayTeam = function() {
    return this.title.slice(this.title.indexOf(' v ')+3);
  }

  Match.prototype.matchDate = function() {
    return this.startsAt.format('YYYY-MM-DD');
  };

  Match.prototype.matchDateCalendar = function() {
    return this.startsAt.calendar();
  };

  Match.prototype.startTime = function() {
    return this.startsAt.format('HH:mm');
  };

  return Match;
}])

models.factory('Tip', ['Settings', function(Settings) {
  function Tip(data) {
    this.id = data.id;
    this.tipster = data.tipster;
    this.match = data.match;
    this.minute = data.minute;
    this.homeGoals = data.homeGoals;
    this.awayGoals = data.awayGoals;
    this.placedAt = moment(data.placedAt).tz(Settings.timezone);
    this.betTypeId = data.betTypeId;
    this.details = data.details;
  }

  Tip.prototype.title = function() {
    switch (this.betTypeId) {
      case 1: return "Asian Handicap";
      case 2: return "Match Odds";
      case 3: return "Correct Score";
      case 4: return "First Half Goals";
      case 5: return "Over/Under";
      case 6: return "Draw No Bet";
      case 7: return "Clean Sheet";
      case 8: return "Half With Most Goals";
      case 9: return "Half Time/Full Time";
      case 10: return "Winner 2012";
      case 11: return "To Qualify";
      case 12: return "Both Teams To Score?";
      case 13: return "Asian Handicap 1st Half";
      case 14: return "Over/Under 1st Half";
      case 15: return "Match Odds 1st Half";
      case 16: return "Team Over/Under";
      case 17: return "Team Over/Under 1st Half";
      default: return "UNKNOWN BET TYPE";
    }
  };

  Tip.prototype.isLive = function() {
    return !(this.minute == null);
  }

  Tip.prototype.formattedUnits = function() {
    if (!this.details) return null;
    return numeral(this.details.stake).format('0.[00]');
  }

  Tip.prototype.hasMotivation = function() {
    if (this.details && this.details.motivation && this.details.motivation.length > 0) return true;
    return false;
  }

  Tip.prototype.motivationHtml = function() {
    if (!this.hasMotivation()) {
      return null;
    };
    function replaceLinebreak(match) {
      return '<br />';
    }
    return this.details.motivation.replace(/\r/g, replaceLinebreak);
  }

  return Tip;
}]);

models.factory('Notification', ['Settings', function(Settings) {
  function Notification(data) {
    this.id = data.id;
    this.title = data.title;
    this.postedAt = moment(data.postedAt).tz(Settings.timezone);
    this.type = data.type;
    this.message = data.message;
    this.url = data.url;
  }

  Notification.prototype.externalLink = function(title, cssClass) {
    return "<a class='" + cssClass + "' href='#' onclick='window.open(\"" + this.url + "\", \"_system\"); return false;'>" + title + "</a>";
  };

  return Notification;
}]);

models.factory('Tipster', ['Settings', function(Settings) {
  function Tipster(data, fromDate) {
    this.id = data.id;
    this.name = data.name;
    this.description = data.description;
    this.avatar = data.avatar;
    this.twitterUsername = data.twitterUsername;
    this.numberOfTips = data.numberOfTips;
    this.numberOfTipsPrematch = data.numberOfTipsPrematch;
    this.numberOfTipsLive = data.numberOfTipsLive;
    this.numberOfTipsMotivated = data.numberOfTipsMotivated;
    this.units = data.units;
    this.avgPrice = data.avgPrice;
    this.numberWon = parseInt(data.numberWon);
    this.numberHalfwon = parseInt(data.numberHalfwon);
    this.numberLost = parseInt(data.numberLost);
    this.numberHalflost = parseInt(data.numberHalflost);
    this.numberPush = parseInt(data.numberPush);
    this.pnl = data.pnl;
    this.fromDate = fromDate.tz(Settings.timezone);
  }

  Tipster.prototype.hitRate = function() {
    if (this.numberOfTips == 0) { return 0; }
    return ((this.numberWon + (this.numberHalfwon/2) + (this.numberHalflost/2)) / this.numberOfTips) * 100;
  }

  Tipster.prototype.yield = function() {
    if (this.numberOfTips == 0) { return ''; }

		var units = parseFloat(this.units);
		var profit = parseFloat(this.pnl);
		var roi = (units + profit) / units;

		return roi*100 - 100; // Math.round(roi*100)/100;
  }

  return Tipster;
}])
