services = angular.module('futd.services', []);

services.factory(("ionPlatform"), function($q) {
  var ready = $q.defer();

  ionic.Platform.ready(function(device){
    ready.resolve(device);
  });

  return {
    ready: ready.promise
  }
})

services.factory('Api', ['$localstorage', '$base64', '$http', '$state', function($localstorage, $base64, $http, $state) {
  var serverUrl = "https://futd-rest.herokuapp.com";
  // var serverUrl = "http://localhost:5000";
  var apiBasePath = "/api/1";
  var tokenEndpoint = "/oauth/token";
  var clientId = 'YviyJR0VSHt';
  var clientSecret = '0cCq5XwmgdN';

  var _accessToken = function(username, password, done) {
    var method = 'POST';
    var url = serverUrl + tokenEndpoint;

    console.log(method + ' ' + url + ' (ACCESS_TOKEN)');

    var authData = $base64.encode(clientId + ':' + clientSecret);
    var req = {
      method: method,
      url: url,
      headers: {
        'Authorization': 'Basic ' + authData
      },
      data: {grant_type: 'password', username: username, password: password}
    };

    $http(req)
      .success(function(data, status, headers, config) {
        if (data && data.access_token && data.refresh_token && data.expires_in) {
          $localstorage.set('accessToken', data.access_token);
          $localstorage.set('refreshToken', data.refresh_token);
          $localstorage.set('tokenExpiresAt', data.expires_in);
          done(true);
        } else {
          done(false);
        }
      })
      .error(function(data, status, headers, config) {
        console.log("ERROR: Login failed with status " + status);
        done(false, data);
      });
  };

  var _refresh = function(done) {
    var method = 'POST';
    var url = serverUrl + tokenEndpoint;

    console.log(method + ' ' + url + ' (REFRESH)');

    var refreshToken = $localstorage.get('refreshToken');
    if (!refreshToken || refreshToken == "undefined") {
      console.log('missing refresh token');
      return false;
    }

    var authData = $base64.encode(clientId + ':' + clientSecret);
    var req = {
      method: method,
      url: url,
      headers: {
        'Authorization': 'Basic ' + authData
      },
      data: {grant_type: 'refresh_token', refresh_token: refreshToken}
    };

    $http(req)
      .success(function(data, status, headers, config) {
        console.log('Refreshed access token');
        $localstorage.set('accessToken', data.access_token);
        $localstorage.set('tokenExpiresAt', data.expires_in);
        return done(true);
      })
      .error(function(data, status, headers, config) {
        console.log('ERROR: Refresh failed with status ' + status);
        return done(false, data);
      });
  };

  var _clearTokens = function() {
    $localstorage.set('accessToken', undefined);
    $localstorage.set('refreshToken', undefined);
    $localstorage.set('tokenExpiresAt', undefined);
  };

  var _request = function(options, done) {
    var method = options.method || 'GET';
    var url = serverUrl + apiBasePath + options.endpoint;
    var requestData = options.data || null;

    console.log(method + ' ' + url);

    var accessToken = $localstorage.get('accessToken');
    if (!accessToken || accessToken == "undefined") {
      console.log('missing access token');
      return done(false, {error: 'token_missing'}, 0);
    }
    // TODO: Check expiry date

    var req = {
      method: method,
      url: url,
      headers: {
        'Authorization': 'Bearer ' + accessToken
      },
      data: requestData
    };

    $http(req)
      .success(function(data, status, headers, config) {
        return done(true, data, status);
      })
      .error(function(data, status, headers, config) {
        if (status === 401) {
          challenge = headers('WWW-Authenticate');
          if (challenge && challenge.indexOf('invalid_token') > -1) {
            console.log(challenge);

            _refresh(function(success) {
              if (success) {
                return _request(options, done);
              } else {
                return done(false, {error: 'invalid_token'}, status);
              }
            });
          } else {
            return done(false, data, status);
          }
        } else {
          return done(false, data, status);
        }
      });
  };

  return {
    accessToken: _accessToken,
    refresh: _refresh,
    clearTokens: _clearTokens,
    request: _request
  }
}]);

services.factory('$localstorage', ['$window', function($window) {
  return {
    set: function(key, value) {
      $window.localStorage[key] = value;
    },
    get: function(key, defaultValue) {
      return $window.localStorage[key] || defaultValue;
    },
    setObject: function(key, value) {
      $window.localStorage[key] = JSON.stringify(value);
    },
    getObject: function(key) {
      return JSON.parse($window.localStorage[key] || '{}');
    }
  }
}]);

services.factory('Users', ['$state', '$cordovaToast', '$cordovaDialogs', 'Settings', 'Api', 'User', function($state, $cordovaToast, $cordovaDialogs, Settings, Api, User) {
  return {
    currentUser: function(done) {
      Api.request({endpoint: '/user'}, function (success, data, status) {
        if (success) {
          var u = new User(data);
          Settings.timezone = u.timezone;
          return done(u);
        }

        if (data && data.error && data.error == 'token_missing') {
          $state.go('signin');
        } else if (data && data.error && (data.error == 'invalid_token')) {
          console.log('No active session on server');
          if (ionic.Platform.isWebView()) {
            $cordovaToast.showLongCenter('Your session has expired. Please sign in again.');
          }
          Api.clearTokens();
          $state.go('signin');
        } else {
          console.log('ERROR: Request for /user failed.');
          console.log('HTTP Status: ' + status);
          console.log(data);
          if (ionic.Platform.isWebView()) {
            $cordovaDialogs.alert('An error occurred while communicating with the server.', 'Error');
          }
        }
      });
    },
    update: function(user, done) {
      Api.request({method: 'PUT', endpoint: '/user', data: user}, function(success) {
        done(success);
      });
    }
  }
}]);

services.factory('Devices', ['Api', function(Api) {
  return {
    registerAppleDevice: function(deviceToken, done) {
      Api.request({method: 'PUT', endpoint: '/devices/ios/'+deviceToken}, function (success) {
        if (success) {
          return done(true);
        } else {
          return done(null);
        }
      });
    },
    registerAndroidDevice: function(regid, done) {
      Api.request({method: 'PUT', endpoint: '/devices/android/'+regid}, function(success) {
        if (success) {
          return done(true);
        } else {
          return done(null);
        }
      });
    }
  }
}]);

services.factory('Notifications', ['Api', 'Notification', function(Api, Notification) {
  return {
    list: function(done) {
      Api.request({endpoint: '/notifications'}, function(success, data) {
        if (success) {
          var notifications = data.map(function(n) {
            return new Notification(n);
          });
          return done(notifications);
        } else {
          return done(null);
        }
      });
    }
  }
}]);

services.factory('Matches', ['Api', 'Match', 'ListItem', 'Ad', function(Api, Match, ListItem, Ad) {
  return {
    groupByDate: function(done) {
      Api.request({endpoint: '/matches?state=upcoming'}, function(success, data) {
        if (success) {
          var items = data.map(function(m, i) {
            var tips = new Array(parseInt(m.tipcount));
            var match = new Match(m.id, m.title, m.startsAt, m.competition, tips);
            return new ListItem('match', match, match.matchDate(), match.startTime());
          });

          if (items.length > 1) {
            Api.request({endpoint: '/ad'}, function(success, data) {
              if (success && data) {
                var ad = new Ad(data.url, data.img);
                items.splice(1, 0, new ListItem('ad', ad, items[0].inner.matchDate(), items[0].inner.startTime()));
              }
            });
          }

          return done(items);
        } else {
          return done(null);
        }
      });
    },
    live: function(done) {
      Api.request({endpoint: '/matches?state=live'}, function(success, data) {
        if (success) {
          var matches = data.map(function(m) {
            var tips = new Array(parseInt(m.tipcount));
            return new Match(m.id, m.title, m.startsAt, m.competition, tips);
          });
          return done(matches);
        } else {
          return done(null);
        }
      });
    },
    get: function(matchId, done) {
      Api.request({endpoint: '/matches/'+matchId}, function(success, data) {
        if (success) {
          var match = new Match(data.id, data.title, data.startsAt, data.competition, data.tips);
          return done(match)
        } else {
          return done(null);
        }
      });
    }
  }
}]);

services.factory('Tips', ['Api', 'Tip', 'ListItem', 'Ad', function(Api, Tip, ListItem, Ad) {
  return {
    live: function(done) {
      Api.request({endpoint: '/tips/live'}, function(success, data) {
        if (success) {
          console.log(data);
          var items = data.map(function(t) {
            var tip = new Tip(t);
            return new ListItem('tip', tip);
          });

          if (items.length > 1) {
            Api.request({endpoint: '/ad?view=live'}, function(success, data) {
              if (success && data) {
                var ad = new Ad(data.url, data.img);
                items.splice(0, 0, new ListItem('ad', ad));
              }
            });
          }

          return done(items);
        } else {
          return done(null);
        }
      });
    }
  }
}]);

services.factory('Tipsters', ['Api', 'Tipster', function(Api, Tipster) {
  return {
    top: function(done) {
      var fromDate = moment().subtract(30, 'days');
      Api.request({endpoint: '/tipsters/top'}, function(success, data) {
        if (success) {
          var tipsters = data.map(function(t) {
            return new Tipster(t, fromDate);
          });
          return done(tipsters);
        } else {
          return done(null);
        }
      });
    },
    all: function(done) {
      var fromDate = moment('2011-01-01');
      Api.request({endpoint: '/tipsters'}, function(success, data) {
        if (success) {
          var tipsters = data.map(function(t) {
            return new Tipster(t, fromDate);
          });
          return done(tipsters);
        } else {
          return done(null);
        }
      });
    },
    get: function(tipsterId, done) {
      var fromDate = moment().subtract(30, 'days');
      Api.request({endpoint: '/tipsters/' + tipsterId}, function(success, data) {
        if (success) {
          var tipster = new Tipster(data, fromDate)
          return done(tipster);
        } else {
          return done(false);
        }
      });
    }
  }
}]);
